@@ -15,13 +15,13 @@ Net::SNTP::Client - Perl module to calculate the roundtrip delay d and
 
 =head1 VERSION
 
-Version 0.12
+Version 0.13
 
 =cut
 
 ## Version of the Net::SNTP::Client module
 
-our $VERSION = '0.12';
+our $VERSION = '0.13';
 $VERSION = eval $VERSION;
 
 ## Load our modules
@@ -189,25 +189,23 @@ sub getSNTPTime {
 	"Transmit Timestamp Micro Sec"  => 0,
 	);
 
-    my @Array_Hash_To_Receive = (	
-	{ "LI VN Mode"                    => '0' },        # 8 bit
-	{ "Stratum"                       => '0' },        # 8 bit
-	{ "Poll"                          => '0' },        # 8 bit
-	{ "Precision"                     => '0' },        # 8 bit
-	{ "Root Delay"                    => '0' },        # 16 bit
-	{ "Root Delay Fraction"           => '0' },        # 16 bit
-	{ "Root Dispersion"               => '0' },        # 16 bit
-	{ "Root Dispersion Fraction"      => '0' },        # 16 bit
-	{ "Reference Identifier"          => '0' },        # 32 bit
-	{ "Reference Timestamp Sec"       => '0' },        # 32 bit
-	{ "Reference Timestamp Micro Sec" => '0' },        # 32 bit
-	{ "Originate Timestamp Sec"       => '0' },        # 32 bit
-	{ "Originate Timestamp Micro Sec" => '0' },        # 32 bit
-	{ "Receive Timestamp Sec"         => '0' },        # 32 bit
-	{ "Receive Timestamp Micro Sec"   => '0' },        # 32 bit
-	{ "Transmit Timestamp Sec"        => '0' },        # 32 bit
-	{ "Transmit Timestamp Micro Sec"  => '0' },        # 32 bit
-	);
+    my @SNTP_Receive = ( "LI VN Mode",
+			 "Stratum",
+			 "Poll",
+			 "Precision",
+			 "Root Delay",
+			 "Root Delay Fraction",
+			 "Root Dispersion",
+			 "Root Dispersion Fraction",
+			 "Reference Identifier",
+			 "Reference Timestamp Sec",
+			 "Reference Timestamp Micro Sec",
+			 "Originate Timestamp Sec",
+			 "Originate Timestamp Micro Sec",
+			 "Receive Timestamp Sec",
+			 "Receive Timestamp Micro Sec",
+			 "Transmit Timestamp Sec",
+			 "Transmit Timestamp Micro Sec" );
 
     ( $SNTP_Client_Hash{"Transmit Timestamp Sec"} , $SNTP_Client_Hash{"Transmit Timestamp Micro Sec"} ) = gettimeofday();
     
@@ -223,12 +221,22 @@ sub getSNTPTime {
     };
     return ($error = "Problem While Sending '$!'", \%moduleInput) if ( $@ && $@ =~ /Error Sending/ );
 
+    my $TIMEOUT = 20;
     my $rcvSntpPacket = undef;
     eval {
+	local $SIG{ALRM} = sub { die "Error Timeout"; };
+	alarm($TIMEOUT);
 	$client_socket->recv( $rcvSntpPacket , MAXBYTES )
 	    or die "Error Receiving";
+	alarm(0)
     };
-    return ($error = "Problem While Receiving '$!'", \%moduleInput) if ( $@ && $@ =~ /Error Receiving/ );
+
+    if ( $@ && $@ =~ /Error Receiving/ ){
+	return ($error = "Problem While Receiving '$!'", \%moduleInput);
+    }
+    elsif ($@ && $@ =~ /Error Timeout/) {
+	return ($error = "Net::SNTP::Client timed out waiting the packet '$!'", \%moduleInput);
+    }
 
     ( $SNTP_Client_Hash{"Receive Timestamp Sec"} , $SNTP_Client_Hash{"Receive Timestamp Micro Sec"} ) = gettimeofday();
 
@@ -238,76 +246,55 @@ sub getSNTPTime {
     };
     return ($error = "Problem While Clossing Socket '$!'", \%moduleInput) if ( $@ && $@ =~ /Error Closing Socket/ );
 
-    my @arrayRcvSntpPacket = unpack("B8 C3 s B16 n B16 H8 N8" , $rcvSntpPacket);
+    my %RcV;
+    @RcV{@SNTP_Receive} = unpack("B8 C3 s B16 n B16 H8 N8" , $rcvSntpPacket);
 
-    my $i = 0;
-    foreach my $hash_ref ( @Array_Hash_To_Receive ) {
-	foreach my $role_hash ( keys %$hash_ref ) {
-	    $hash_ref->{$role_hash} = $arrayRcvSntpPacket[$i++];
-	}
-    }
-    $i = undef;
+    $RcV{"LI Binary"} = substr( $RcV{"LI VN Mode"} , 0 , 2 );
+    $RcV{"LI"} = _binaryToDecimal( $RcV{"LI Binary"} , 8 , "c" );
+    delete $RcV{"LI Binary"};
 
-    $Array_Hash_To_Receive[0]{"LI Binary"} = 
-	substr( $Array_Hash_To_Receive[0]{"LI VN Mode"} , 0 , 2 );
-    $Array_Hash_To_Receive[0]{"LI"} = 
-	_binaryToDecimal( $Array_Hash_To_Receive[0]{"LI Binary"} , 8 , "c" );
-    delete $Array_Hash_To_Receive[0]{"LI Binary"};
+    $RcV{"VN Binary"} = substr( $RcV{"LI VN Mode"} , 2 , 3 );
+    $RcV{"VN"} = _binaryToDecimal( $RcV{"VN Binary"} , 8 , "c" );
+    delete $RcV{"VN Binary"};
 
-    $Array_Hash_To_Receive[0]{"VN Binary"} = 
-	substr( $Array_Hash_To_Receive[0]{"LI VN Mode"} , 2 , 3 );
-    $Array_Hash_To_Receive[0]{"VN"} = 
-	_binaryToDecimal( $Array_Hash_To_Receive[0]{"VN Binary"} , 8 , "c" );
-    delete $Array_Hash_To_Receive[0]{"LI Binary"};
+    $RcV{"Mode Binary"} = substr( $RcV{"LI VN Mode"} , 5 , 3 );
+    $RcV{"Mode"} = _binaryToDecimal( $RcV{"Mode Binary"} , 8 , "c" );
+    delete $RcV{"Mode Binary"};
+    delete $RcV{"LI VN Mode"};
 
-    $Array_Hash_To_Receive[0]{"Mode Binary"} = 
-	substr( $Array_Hash_To_Receive[0]{"LI VN Mode"} , 5 , 3 );
-    $Array_Hash_To_Receive[0]{"Mode"} = 
-	_binaryToDecimal( $Array_Hash_To_Receive[0]{"Mode Binary"} , 8 , "c" );
-    delete $Array_Hash_To_Receive[0]{"Mode Binary"};
-    delete $Array_Hash_To_Receive[0]{"LI VN Mode"};
+    $RcV{"Poll"} = (sprintf("%.1d", $RcV{"Poll"}));
 
-    $Array_Hash_To_Receive[2]{"Poll"} = 
-	(sprintf("%.1d", $Array_Hash_To_Receive[2]{"Poll"}));
-
-    if ($Array_Hash_To_Receive[3]{"Precision"} < 127) {
-	$Array_Hash_To_Receive[3]{"Precision"} = 0;
+    if ($RcV{"Precision"} < 127) {
+	$RcV{"Precision"} = 0;
     }
     else {
-	$Array_Hash_To_Receive[3]{"Precision"} = $Array_Hash_To_Receive[3]{"Precision"} - 255;
+	$RcV{"Precision"} = $RcV{"Precision"} - 255;
     }
 
-    $Array_Hash_To_Receive[4]{"Root Delay"} = 
-	_binaryToDecimal( $Array_Hash_To_Receive[4]{"Root Delay"} , 16 , "s" );
+    $RcV{"Root Delay"} = 
+	_binaryToDecimal( $RcV{"Root Delay"} , 16 , "s" );
 
-    # print "Root Delay Fraction Binary: " . $Array_Hash_To_Receive[5]{Root_Delay_Fraction} . "\n";
-    $Array_Hash_To_Receive[5]{"Root Delay Fraction"} = 
-	_binaryToDecimal( $Array_Hash_To_Receive[5]{"Root Delay Fraction"} , 16 , "n" );
+    $RcV{"Root Delay Fraction"} = 
+	_binaryToDecimal( $RcV{"Root Delay Fraction"} , 16 , "n" );
     
-    $Array_Hash_To_Receive[4]{"Root Delay"} = 
-	$Array_Hash_To_Receive[4]{"Root Delay"} . "." . $Array_Hash_To_Receive[5]{"Root Delay Fraction"};
+    $RcV{"Root Delay"} = 
+	$RcV{"Root Delay"} . "." . $RcV{"Root Delay Fraction"};
 
-    $Array_Hash_To_Receive[6]{"Root Dispersion"} = 
-	_binaryToDecimal( $Array_Hash_To_Receive[6]{"Root Dispersion"} , 16 , "n" );
+    $RcV{"Root Dispersion"} = 
+	_binaryToDecimal( $RcV{"Root Dispersion"} , 16 , "n" );
 
-    # print "Root Dispersion Binary: " . $Array_Hash_To_Receive[7]{Root_Dispersion_Fraction} . "\n";
-    $Array_Hash_To_Receive[7]{"Root Dispersion Fraction"} = 
-	_binaryToDecimal( $Array_Hash_To_Receive[7]{"Root Dispersion Fraction"} , 16 , "n" );
+    $RcV{"Root Dispersion Fraction"} = 
+	_binaryToDecimal( $RcV{"Root Dispersion Fraction"} , 16 , "n" );
     
-    $Array_Hash_To_Receive[6]{"Root Dispersion"} = 
-	$Array_Hash_To_Receive[6]{"Root Dispersion"} . "." . $Array_Hash_To_Receive[7]{"Root Dispersion Fraction"};
+    $RcV{"Root Dispersion"} = 
+	$RcV{"Root Dispersion"} . "." . $RcV{"Root Dispersion Fraction"};
     
-    $Array_Hash_To_Receive[8]{"Reference Identifier"} = 
-	_unpackIP($Array_Hash_To_Receive[1]{"Stratum"},$Array_Hash_To_Receive[8]{"Reference Identifier"});
+    $RcV{"Reference Identifier"} = 
+	_unpackIP($RcV{"Stratum"},$RcV{"Reference Identifier"});
 
-    $Array_Hash_To_Receive[9]{"Reference Timestamp Sec"} = 
-	$Array_Hash_To_Receive[9]{"Reference Timestamp Sec"} - UNIX_EPOCH;
-
-    $Array_Hash_To_Receive[13]{"Receive Timestamp Sec"} = 
-	$Array_Hash_To_Receive[13]{"Receive Timestamp Sec"} - UNIX_EPOCH;
-
-    $Array_Hash_To_Receive[15]{"Transmit Timestamp Sec"} = 
-	$Array_Hash_To_Receive[15]{"Transmit Timestamp Sec"} - UNIX_EPOCH;
+    $RcV{"Reference Timestamp Sec"} -= UNIX_EPOCH;
+    $RcV{"Receive Timestamp Sec"} -= UNIX_EPOCH;
+    $RcV{"Transmit Timestamp Sec"} -= UNIX_EPOCH;
 
     my $d = (
 	(
@@ -315,19 +302,19 @@ sub getSNTPTime {
 	 ( $SNTP_Client_Hash{"Transmit Timestamp Sec"} . "." . $SNTP_Client_Hash{"Transmit Timestamp Micro Sec"} ) 
 	) -
 	(
-	 ( $Array_Hash_To_Receive[15]{"Transmit Timestamp Sec"} . "." . $Array_Hash_To_Receive[16]{"Transmit Timestamp Micro Sec"} ) -
-	 ( $Array_Hash_To_Receive[13]{"Receive Timestamp Sec"} . "." . $Array_Hash_To_Receive[14]{"Receive Timestamp Micro Sec"} )
+	 ( $RcV{"Transmit Timestamp Sec"} . "." . $RcV{"Transmit Timestamp Micro Sec"} ) -
+	 ( $RcV{"Receive Timestamp Sec"} . "." . $RcV{"Receive Timestamp Micro Sec"} )
 	) 
 	);
 
     my $t = (
 	(
 	 (
-	  ( $Array_Hash_To_Receive[13]{"Receive Timestamp Sec"} . "." . $Array_Hash_To_Receive[14]{"Receive Timestamp Micro Sec"} ) -
+	  ( $RcV{"Receive Timestamp Sec"} . "." . $RcV{"Receive Timestamp Micro Sec"} ) -
 	  ( $SNTP_Client_Hash{"Transmit Timestamp Sec"} . "." . $SNTP_Client_Hash{"Transmit Timestamp Micro Sec"} ) 
 	 ) +
 	 (
-	  ( $Array_Hash_To_Receive[15]{"Transmit Timestamp Sec"} . "." . $Array_Hash_To_Receive[16]{"Transmit Timestamp Micro Sec"} ) -
+	  ( $RcV{"Transmit Timestamp Sec"} . "." . $RcV{"Transmit Timestamp Micro Sec"} ) -
 	  ( $SNTP_Client_Hash{"Receive Timestamp Sec"} . "." . $SNTP_Client_Hash{"Receive Timestamp Micro Sec"} )
 	 )
 	) / 2
@@ -339,67 +326,67 @@ sub getSNTPTime {
     
     if ( $moduleInput{-RFC4330} ) {
 	$moduleOutput{-RFC4330} = " 
-  \t Timestamp Name \t ID \t When Generated
-  \t ------------------------------------------------------------
-  \t Originate Timestamp \t T1 \t time request sent by client
-  \t Receive Timestamp \t T2 \t time request received by server
-  \t Transmit Timestamp \t T3 \t time reply sent by server
-  \t Destination Timestamp \t T4 \t time reply received by client
+\t Timestamp Name \t ID \t When Generated
+\t ------------------------------------------------------------
+\t Originate Timestamp \t T1 \t time request sent by client
+\t Receive Timestamp \t T2 \t time request received by server
+\t Transmit Timestamp \t T3 \t time reply sent by server
+\t Destination Timestamp \t T4 \t time reply received by client
 
-  \t The roundtrip delay d and local clock offset t are defined as
+\t The roundtrip delay d and local clock offset t are defined as
 
-  \t d = (T4 - T1) - (T2 - T3) \t t = ((T2 - T1) + (T3 - T4)) / 2 \n
+\t d = (T4 - T1) - (T2 - T3) \t t = ((T2 - T1) + (T3 - T4)) / 2 \n
 
-  \t Round Trip delay: ".$d."\n
-  \t Clock offset: ".$t."\n
+\t Round Trip delay: ".$d."\n
+\t Clock offset: ".$t."\n
 
-  \t Field Name \t\t\t Unicast/Anycast
-  \t\t\t\t Request \t\t Reply
-  \t ------------------------------------------------------------
-  \t LI \t\t\t ".$SNTP_Client_Hash{"LI"}." \t\t\t ".$Array_Hash_To_Receive[0]{"LI"}."
-  \t VN \t\t\t ".$SNTP_Client_Hash{"VN"}." \t\t\t ".$Array_Hash_To_Receive[0]{"VN"}."
-  \t Mode \t\t\t ".$SNTP_Client_Hash{"Mode"}." \t\t\t ".$Array_Hash_To_Receive[0]{"Mode"}."
-  \t Stratum \t\t ".$SNTP_Client_Hash{"Stratum"}." \t\t\t ".$Array_Hash_To_Receive[1]{"Stratum"}."
-  \t Poll \t\t\t ".$SNTP_Client_Hash{"Poll"}." \t\t\t ".$Array_Hash_To_Receive[2]{"Poll"}."
-  \t Precision \t\t ".$SNTP_Client_Hash{"Precision"}." \t\t\t ".$Array_Hash_To_Receive[3]{"Precision"}."
-  \t Root Delay \t\t ".$SNTP_Client_Hash{"Root Delay"}." \t\t\t ".$Array_Hash_To_Receive[4]{"Root Delay"}."
-  \t Root Dispersion \t ".$SNTP_Client_Hash{"Root Dispersion"}." \t\t\t ".$Array_Hash_To_Receive[6]{"Root Dispersion"}."
-  \t Reference Identifier \t ".$SNTP_Client_Hash{"Reference Identifier"}." \t\t\t ".$Array_Hash_To_Receive[8]{"Reference Identifier"}."
-  \t Reference Timestamp \t ".$SNTP_Client_Hash{"Reference Timestamp"}." \t\t\t ".
-	    $Array_Hash_To_Receive[9]{"Reference Timestamp Sec"}.".".
-	    $Array_Hash_To_Receive[10]{"Reference Timestamp Micro Sec"}."
-  \t Originate Timestamp \t ".$SNTP_Client_Hash{"Originate Timestamp"}." \t\t\t ".
-	    $Array_Hash_To_Receive[11]{"Originate Timestamp Sec"}.".".
-	    $Array_Hash_To_Receive[12]{"Originate Timestamp Micro Sec"}."
-  \t Receive Timestamp \t ".
+\t Field Name \t\t\t Unicast/Anycast
+\t\t\t\t Request \t\t Reply
+\t ------------------------------------------------------------
+\t LI \t\t\t ".$SNTP_Client_Hash{"LI"}." \t\t\t ".$RcV{"LI"}."
+\t VN \t\t\t ".$SNTP_Client_Hash{"VN"}." \t\t\t ".$RcV{"VN"}."
+\t Mode \t\t\t ".$SNTP_Client_Hash{"Mode"}." \t\t\t ".$RcV{"Mode"}."
+\t Stratum \t\t ".$SNTP_Client_Hash{"Stratum"}." \t\t\t ".$RcV{"Stratum"}."
+\t Poll \t\t\t ".$SNTP_Client_Hash{"Poll"}." \t\t\t ".$RcV{"Poll"}."
+\t Precision \t\t ".$SNTP_Client_Hash{"Precision"}." \t\t\t ".$RcV{"Precision"}."
+\t Root Delay \t\t ".$SNTP_Client_Hash{"Root Delay"}." \t\t\t ".$RcV{"Root Delay"}."
+\t Root Dispersion \t ".$SNTP_Client_Hash{"Root Dispersion"}." \t\t\t ".$RcV{"Root Dispersion"}."
+\t Reference Identifier \t ".$SNTP_Client_Hash{"Reference Identifier"}." \t\t\t ".$RcV{"Reference Identifier"}."
+\t Reference Timestamp \t ".$SNTP_Client_Hash{"Reference Timestamp"}." \t\t\t ".
+	    $RcV{"Reference Timestamp Sec"}.".".
+	    $RcV{"Reference Timestamp Micro Sec"}."
+\t Originate Timestamp \t ".$SNTP_Client_Hash{"Originate Timestamp"}." \t\t\t ".
+	    $RcV{"Originate Timestamp Sec"}.".".
+	    $RcV{"Originate Timestamp Micro Sec"}."
+\t Receive Timestamp \t ".
 	    $SNTP_Client_Hash{"Receive Timestamp Sec"}.".".$SNTP_Client_Hash{"Receive Timestamp Micro Sec"}." \t ".
-	    $Array_Hash_To_Receive[13]{"Receive Timestamp Sec"} . ".". 
-	    $Array_Hash_To_Receive[14]{"Receive Timestamp Micro Sec"}."
-  \t Transmit Timestamp \t ".
+	    $RcV{"Receive Timestamp Sec"} . ".". 
+	    $RcV{"Receive Timestamp Micro Sec"}."
+\t Transmit Timestamp \t ".
 	    $SNTP_Client_Hash{"Transmit Timestamp Sec"} . "." . $SNTP_Client_Hash{"Transmit Timestamp Micro Sec"}." \t ".
-	    $Array_Hash_To_Receive[15]{"Transmit Timestamp Sec"} . ".". 
-	    $Array_Hash_To_Receive[16]{"Transmit Timestamp Micro Sec"}."\n\n";
+	    $RcV{"Transmit Timestamp Sec"} . ".". 
+	    $RcV{"Transmit Timestamp Micro Sec"}."\n\n";
     }
     else {
 	%moduleOutput = (
 	    $moduleInput{-hostname} => {
-		"LI"                   => $Array_Hash_To_Receive[0]{"LI"},
-		    "VN"                   => $Array_Hash_To_Receive[0]{"VN"},
-		    "Mode"                 => $Array_Hash_To_Receive[0]{"Mode"},
-		    "Stratum"              => $Array_Hash_To_Receive[1]{"Stratum"},
-		    "Poll"                 => $Array_Hash_To_Receive[2]{"Poll"},
-		    "Precision"            => $Array_Hash_To_Receive[3]{"Precision"},
-		    "Root Delay"           => $Array_Hash_To_Receive[4]{"Root Delay"},
-		    "Root Dispersion"      => $Array_Hash_To_Receive[6]{"Root Dispersion"},
-		    "Reference Identifier" => $Array_Hash_To_Receive[8]{"Reference Identifier"},
-		    "Reference Timestamp"  => $Array_Hash_To_Receive[9]{"Reference Timestamp Sec"}.".".
-		    $Array_Hash_To_Receive[10]{"Reference Timestamp Micro Sec"},
-		    "Originate Timestamp"  => $Array_Hash_To_Receive[11]{"Originate Timestamp Sec"}.".".
-		    $Array_Hash_To_Receive[12]{"Originate Timestamp Micro Sec"},
-		    "Receive Timestamp"    => $Array_Hash_To_Receive[13]{"Receive Timestamp Sec"}.".".
-		    $Array_Hash_To_Receive[14]{"Receive Timestamp Micro Sec"},
-		    "Transmit Timestamp"   => $Array_Hash_To_Receive[15]{"Transmit Timestamp Sec"}.".".
-		    $Array_Hash_To_Receive[16]{"Transmit Timestamp Micro Sec"},
+		"LI"                   => $RcV{"LI"},
+		    "VN"                   => $RcV{"VN"},
+		    "Mode"                 => $RcV{"Mode"},
+		    "Stratum"              => $RcV{"Stratum"},
+		    "Poll"                 => $RcV{"Poll"},
+		    "Precision"            => $RcV{"Precision"},
+		    "Root Delay"           => $RcV{"Root Delay"},
+		    "Root Dispersion"      => $RcV{"Root Dispersion"},
+		    "Reference Identifier" => $RcV{"Reference Identifier"},
+		    "Reference Timestamp"  => $RcV{"Reference Timestamp Sec"}.".".
+		    $RcV{"Reference Timestamp Micro Sec"},
+		    "Originate Timestamp"  => $RcV{"Originate Timestamp Sec"}.".".
+		    $RcV{"Originate Timestamp Micro Sec"},
+		    "Receive Timestamp"    => $RcV{"Receive Timestamp Sec"}.".".
+		    $RcV{"Receive Timestamp Micro Sec"},
+		    "Transmit Timestamp"   => $RcV{"Transmit Timestamp Sec"}.".".
+		    $RcV{"Transmit Timestamp Micro Sec"},
 	    },
 	    $0 => {
 		"LI"                   => $SNTP_Client_Hash{"LI"},
@@ -637,7 +624,7 @@ EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 =head1 CHANGE LOG
 $Log: Client.pm,v $
-Revision 12.0  2015/07/25 12:10:31 pm  Thanos
+Revision 13.0  2015/07/26 2:34:31 pm  Thanos
 
 
 =cut
